/*************************************************************************************
 
LMS Longest Monotonically Increasing Sequence Algorithm

Author: Eric Breimer
File: lms.cpp

*************************************************************************************/

#include <vector>
#include <iostream>
#include <time.h>
#include <math.h>
#include "lmsmisc.h"
#include "lmsalgo.h"

using std::vector;
using std::cout;
using std::endl;
using std::cerr;

/*************************************************************************************

Print usage

*************************************************************************************/
void print_usage() {
  cout << "Usage: lms n d t r seed" << "\n";
  cout << "  n     number of input points" << "\n";
  cout << "  d     dimensions" << "\n";
  cout << "  t     trials" << "\n";
  cout << "  r     radius" << "\n";
  cout << "  seed  random number seed" << "\n";
}



/*************************************************************************************

Main Program

*************************************************************************************/
int main(int argc, char * argv[]) {

  // Print usage if necessary
  if (argc != 6) {
    print_usage();
    return 1;
  } 

  int N;                           // Number of input points
  int D;                           // Number of dimensions
  int T;                           // Trials
  double R;                        // Radius from the diagonal
  long S;                          // Random number seed

  // Statistics
  clock_t start, duration;
  double Nd, max, min, ave, dev;
  vector<int> lmssum;
  vector<double> timesum;
  int lmssize = 0;
  int x;

  // Set command-line parameters
  N = atoi(argv[1]);      
  D = atoi(argv[2]);
  T = atoi(argv[3]);
  R = atof(argv[4]);
  S = (long) atoi(argv[5]);

  // Set random seed
  srand48(S);

  Nd = computeNd(N,D,R);
  cout << "Input parameters: ";
  cout << "N= " << N << "  ";
  cout << "D= " << D << "  "; 
  cout << "trials= " << T << "  ";
  cout << "radius= " << R << "  ";
  cout << "N^(1/d)= " << Nd << endl;
  
  // Loop for T trials
  for (x = 0; x < T; x++) { 

    start = clock();

    // Find longest monotonically increasing subsequence
    lmssize = lms(N, D, R);

    // Accumulate statistics
    timesum.push_back((double) ((double) (clock() - start))/CLOCKS_PER_SEC);
    lmssum.push_back(lmssize);
  }

  cout << "Stats: \tmax \t\tave \t\tmin \t\tdev" << endl;
  max = computeMax(timesum);
  min = computeMin(timesum);
  ave = computeAve(timesum);
  dev = computeDev(timesum,ave);
  cout << "Time  \t" << max << "\t\t" << ave << "\t\t" << min << "\t\t" << dev << endl;
  max = computeMax(lmssum);
  min = computeMin(lmssum);
  ave = computeAve(lmssum);
  dev = computeDev(lmssum,ave);
  cout << "lms   \t" << max << "\t\t" << ave << "\t\t" << min << "\t\t" << dev << endl;
  max /= Nd;
  min /= Nd;
  ave /= Nd;
  dev /= Nd;
  cout << "C     \t" << max << "\t\t" << ave << "\t\t" << min << "\t\t" << dev << endl;
  cout << endl; 

  // Clear vectors
  timesum.clear();
  lmssum.clear();
 
  return 0;
}

  
